package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.utils.Md5Util;
import cn.wisenergy.common.utils.R;
import cn.wisenergy.mapper.*;
import cn.wisenergy.model.app.*;
import cn.wisenergy.model.dto.StaffAddDto;
import cn.wisenergy.model.dto.UserDto;
import cn.wisenergy.model.dto.UserInfoDto;
import cn.wisenergy.model.enums.LonginType;
import cn.wisenergy.model.vo.StaffLoginVo;
import cn.wisenergy.model.vo.StaffVo;
import cn.wisenergy.model.vo.UserVipQueryVo;
import cn.wisenergy.service.app.StaffUserVipService;
import cn.wisenergy.service.common.UserVipManager;
import cn.wisenergy.service.common.VolunteerManager;
import cn.wisenergy.service.shir.filter.KickoutSessionControlFilter;
import cn.wisenergy.service.util.UserToken;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.IncorrectCredentialsException;
import org.apache.shiro.authc.LockedAccountException;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class StaffUserVipServiceImpl extends ServiceImpl<LocalMapper, Local> implements StaffUserVipService {

    @Autowired
    private StaffMapper staffMapper;

    @Autowired
    private StaffUserVipMapper staffUserVipMapper;

    @Autowired
    private UsersMapper usersMapper;

    @Autowired
    private ScoreInfoMapper scoreInfoMapper;

    @Autowired
    private LoginRecordMapper loginRecordMapper;

    @Autowired
    private UserLimitMapper userLimitMapper;

    @Autowired
    private PayRecordMapper payRecordMapper;

    @Autowired
    private VolunteerManager volunteerManager;

    @Autowired
    private UserVipManager userVipManager;

    @Autowired
    private LocalMapper localMapper;

    @Autowired
    private KickoutSessionControlFilter kickoutSessionControlFilter;

    private static final int ERROR_CODE = 1;

    //初始密码
    private static final String PASSWORD = "123456";
    private static final String STAFF_LOGIN_TYPE = LonginType.STAFF.getDescription();

    @Override
    public R<StaffVo> staffLogin(StaffLoginVo loginVo) {
        log.info("StaffUserVipServiceImpl[].staffLogin[].input:param:" + loginVo);

        //判断参数是否为空
        if (null == loginVo || StringUtils.isBlank(loginVo.getLoginName()) || StringUtils.isBlank(loginVo.getPassword())) {
            return R.error("账号或密码为空");
        }

     /*   //创建参数容器map
        HashMap<String, Object> map = new HashMap<>(4);
        map.put("loginName", loginVo.getLoginName());

        //查询管理者信息
        Staff staff = staffMapper.getStaff(map);
        if (null == staff) {
            return R.error("用户不存在");
        }

        if (!staff.getPassword().equals(Md5Util.digestMD5(loginVo.getPassword()))) {
            return R.error("账号或者密码错误");
        }
        QueryWrapper<Staff> objectQueryWrapper = new QueryWrapper<>();
        objectQueryWrapper.eq("login_name",loginVo.getLoginName());
        Staff staffInfo = staffMapper.selectOne(objectQueryWrapper);
        StaffVo staffVo = new StaffVo();
        staffVo.setId(staffInfo.getId());
        staffVo.setLoginName(staffInfo.getLoginName());
        staffVo.setStaffName(staffInfo.getStaffName());
        staffVo.setPhone(staffInfo.getPhone());
        staffVo.setSex(staffInfo.getSex());
        return R.ok(0, staffVo);*/

        Subject subject = SecurityUtils.getSubject();
        String credentialsSalt = Md5Util.digestMD5(loginVo.getPassword() );
        UserToken userToken = new UserToken(loginVo.getLoginName(), credentialsSalt, STAFF_LOGIN_TYPE);
     
        try {
            subject.login(userToken);
            kickoutSessionControlFilter.changeSession(3);
            QueryWrapper<Staff> objectQueryWrapper = new QueryWrapper<>();
            objectQueryWrapper.eq("login_name",loginVo.getLoginName());
            Staff staffInfo = staffMapper.selectOne(objectQueryWrapper);
            StaffVo staffVo = new StaffVo();
            staffVo.setId(staffInfo.getId());
            staffVo.setLoginName(staffInfo.getLoginName());
            staffVo.setStaffName(staffInfo.getStaffName());
            staffVo.setPhone(staffInfo.getPhone());
            staffVo.setSex(staffInfo.getSex());
            staffVo.setSessionId(subject.getSession().getId().toString());
            return R.ok(0, staffVo);
        } catch (IncorrectCredentialsException e) {
            return R.error(1, "账户密码错误");
        } catch (LockedAccountException e) {
            return R.error(1, "登录失败!请联系管理员");
        } catch (AuthenticationException e) {
            return R.error(1, "用户不存在或账号已被禁用");
        } catch (Exception e) {
            return R.error(1, "系统异常");
        }
    }

    @Override
    public R<PageInfo<UserInfoDto>> getList(UserVipQueryVo queryVo) {
        log.info("StaffUserVipServiceImpl[].getList[].input.param:" + queryVo);
        if (null==queryVo) {
            return R.error("入参为空");
        }

        //创建参数容器
        Map<String, Object> map = new HashMap<>(4);

        //将参数放入容器中

        if (null!=queryVo.getPageNo()||null!=queryVo.getPageSize()){
            map.put("pageNo", (queryVo.getPageNo() - 1) * queryVo.getPageSize());
            map.put("pageSize", queryVo.getPageSize());
        }
        map.put("userName", queryVo.getUserName());
        map.put("vipMobile", queryVo.getVipMobile());
        map.put("staffName", queryVo.getStaffName());
        List<User> list1;
        int count;
        if(null != queryVo.getStaffId() && queryVo.getStaffId()>0){
            map.put("staffId",queryVo.getStaffId());
            list1 =  usersMapper.getStaffVipUser(map);
            count=usersMapper.staffVipUserCount(map);
        }else{
            list1 =  usersMapper.getListQueryVip(map);
            count=usersMapper.getListVipCount(map);
        }
        //查询用户数据

        List<UserInfoDto> list = new ArrayList<>();

        //将集合遍历拿出用户id查询成绩信息
        for (User user : list1) {
            UserInfoDto userInfoDto = new UserInfoDto();
            BeanUtils.copyProperties(user, userInfoDto);
            userInfoDto.setUserId(user.getId());
            list.add(userInfoDto);
        }

        PageInfo<UserInfoDto> info = new PageInfo<>();
        info.setPageSize(null==queryVo.getPageSize()?0:queryVo.getPageSize());
        info.setPageNum(null==queryVo.getPageNo()?0:queryVo.getPageNo());
        info.setTotal(count);

        info.setList(list);

        return R.ok(info);
    }

    @Transactional
    @Override
    public R<Boolean> addVipUser(StaffAddDto staffAddDto) {
        log.info("StaffUserVipServiceImpl[].addUserByPhone[].input.param:" +staffAddDto);
        if(null ==staffAddDto || staffAddDto.getStaffId() == null){
            return R.error("入参为空");
        }
        if(!getUserInfo(staffAddDto.getVipMobile())){
            return R.error("该电话已存在");
        }
        R<Boolean> checkScore = volunteerManager.checkScore(staffAddDto.getScoreInfo());
        if (null != checkScore && checkScore.getCode() == ERROR_CODE) {
            return R.error(checkScore.getMessage());
        }

        User user = new User();
        user.setIsVip(1);
        user.setSchool(staffAddDto.getSchool());
        user.setSex(staffAddDto.getSex());
        user.setExamType(staffAddDto.getStudentType());
        user.setIsDelete(0);
        user.setCreateTime(new Date());
        user.setUpdateTime(new Date());
        user.setUserName(staffAddDto.getUserName());
        user.setVipMobile(staffAddDto.getVipMobile());
        int addUser=usersMapper.add(user);
        if(addUser == 0){
            return R.error("添加失败");
        }
        ScoreInfo scoreInfo = staffAddDto.getScoreInfo();
        scoreInfo.setCreateTime(new Date());
        scoreInfo.setUserId(user.getId());
        scoreInfo.setUpdateTime(new Date());
        int addScore=scoreInfoMapper.insert(scoreInfo);
        if(addScore == 0){
            return R.error("添加失败");
        }
        StaffUserVip staffUserVip = new StaffUserVip();
        staffUserVip.setIsDelete(0);
        staffUserVip.setStaffId(staffAddDto.getStaffId());
        staffUserVip.setUserVipId(user.getId());
        staffUserVip.setCreateTime(new Date());
        staffUserVip.setUpdateTime(new Date());
        int insert =  staffUserVipMapper.insert(staffUserVip);
        if(insert>0){
            return R.ok(0,true);
        }
        return R.error("添加失败");
    }

    @Override
    public R<List<Local>> getLocal() {
        List<Local> list = localMapper.getList();

        if (CollectionUtils.isEmpty(list)) {
            return R.error("入参为空");
        }
        return R.ok(0,list);
    }

    @Override
    public R<Boolean> addUserVolunteer(String ids,Integer userId,String name) {
        log.info("StaffUserVipServiceImpl[].addUserVolunteer[].input.param:" +ids,userId);

        if (StringUtils.isBlank(ids)||null==userId) {
            return R.error("入参为空");
        }

        //通过id获取用户信息
        User byId = usersMapper.getById(userId);

        if (null==byId) {
            return R.error("该用户不存在");
        }

        ScoreInfo byUserId = scoreInfoMapper.getByUserId(userId);

        R<Boolean> booleanR = userVipManager.saveUserVolunteer(byId,ids,byUserId,name);

        if (!booleanR.getData()) {
            return R.error("保存失败");
        }
        return R.ok(0,true);
    }



    public boolean getUserInfo(String vipMobile){
        Map<String, Object> map = new HashMap<>(16);
        map.put("vip_mobile",vipMobile);
        User user = usersMapper.selectOne(map);
        if(null == user){
            return true;
        }
        return false;
    }

    @Override
    public R<Boolean> downExcel(UserVipQueryVo queryVo, HttpServletResponse response) {
        log.info("StaffUserVipServiceImpl[].downExcel[].input.param:"+queryVo);
        if (null==queryVo||null==queryVo.getStaffId()) {
            return R.error("入参为空");
        }

        queryVo.setPageSize(null);
        queryVo.setPageNo(null);

        //获取员工名称
        Staff byId = staffMapper.getById(queryVo.getStaffId());
        if (null==byId) {
            return R.error("该员工不存在");
        }
        String staffName=byId.getStaffName();

        //生成Excel
        try {
            //  设置内容格式 以及 编码方式
            response.setContentType("application/vnd.ms-excel");
            response.setCharacterEncoding("utf-8");

            List<UserInfoDto> list=getList(queryVo).getData().getList();

            //  使用java8新特性的stream流去处理数据，把空的数据过滤掉
            List<UserDto> resultBo = list.stream().filter(Objects::nonNull)
                    .map(t -> {
                        return UserDto.builder()
                                .registerTime(t.getRegisterTime())
                                .source(t.getSource()==1?"PC":"APP")
                                .userName(t.getUserName())
                                .lastLoginTime(t.getLastLoginTime())
                                .school(t.getSchool())
                                .examType(t.getExamType()==1?"文化生":t.getExamType()==2?"美术生":t.getExamType()==3?"体育生":"文学编导考生")
                                .sex(t.getSex()==0?"男":"女")
                                .rechargeTimes(t.getRechargeTimes())
                                .weChatMoney(t.getWeChatMoney())
                                .alipayMoney(t.getAlipayMoney())
                                .lastLoginTime(t.getLastLoginTime())
                                .cultureGrade(t.getCultureGrade())
                                .majorGrade(t.getMajorGrade())
                                .build();
                    }).collect(Collectors.toList());

            //创建文件名称
            Long lon = System.currentTimeMillis();

            response.setHeader("Content-disposition", "attachment;filename=" + lon + ".xlsx");

            //  sheet名称
            EasyExcel.write(response.getOutputStream(), UserDto.class).sheet(staffName+lon.toString()).doWrite(resultBo);
            return R.ok(0,true);
        } catch (Exception e) {
            return R.ok(1,false);
        }
    }
}